<?php
/**
 *
 * @category        include
 * @package         localfont
 * @author          Ruud Eisinga - Dev4me
 * @link			http://www.dev4me.nl/
 * @license         http://www.gnu.org/licenses/gpl.html
 * @platform        WebsiteBaker 2.10.x || WBCE 1.4.x
 * @requirements    PHP 7.4 and higher
 * @version         0.1
 * @lastmodified    August 9, 2022
 *
 */

if(defined('WB_PATH') == false) { exit("Cannot access this file directly"); }

global $font;
if(!is_array($font)) return;

$cssdir = '/css/';
$fontdir = '/fonts/';

define ('LOCALFONT_INCLUDE_DEBUG', false);
define ('LOCALFONT_INCLUDE_CSS',  TEMPLATE_DIR . $cssdir);
define ('LOCALFONT_INCLUDE_FONT', TEMPLATE_DIR . $fontdir);
define ('LOCALFONT_PATH_CSS',     __DIR__ . $cssdir);
define ('LOCALFONT_PATH_FONT',    __DIR__ . $fontdir);


$localfonts_include = new localfonts_include();
$rval = "<!-- google fonts loaded locally by 'localfonts_include' - dev4me.com -->".PHP_EOL;
foreach($font as $fontfile) {
	$rval .= "\t".$localfonts_include->get_font_file($fontfile);
}
echo $rval;


class localfonts_include {
	
	function __construct() {
		$this->create_folders();
	}

	function get_font_file($url) {
		
		$force_reload = false;
		if(isset($_SESSION['USER_ID'])) {  //Detect CTRL-F5 only when logged in user
			if(isset($_SERVER['HTTP_CACHE_CONTROL']) && $_SERVER['HTTP_CACHE_CONTROL'] === 'no-cache') {
				$force_reload = true;
				$this->_localfont_log("CTRL-F5 detected, force reloading font: $url");
			}
		}
		
		if($cssfilename = $this->local_name($url)) {
			$rval = '<link href="'.LOCALFONT_INCLUDE_CSS.$cssfilename.'" rel="stylesheet" type="text/css">';
			if(file_exists(LOCALFONT_PATH_CSS.$cssfilename) && !$force_reload) {
				$data = $this->read_font_file($url); // read local css file
			} else {
				$data = $this->read_font_url($url); // read css from google fonts
				preg_match_all("/url\((.*?)\\)/", $data, $matches); 
				foreach($matches[1] as $match) {
					$p = pathinfo($match);
					$filename = $p['filename'].'.'.$p['extension'];
					$data = str_replace($match, LOCALFONT_INCLUDE_FONT.$filename, $data);
					$fontdata = $this->read_font_url($match);
					$this->save_font_file(LOCALFONT_PATH_FONT.$filename,$fontdata);
				}
				$this->save_font_file(LOCALFONT_PATH_CSS.$cssfilename,$data);
			}
			if(LOCALFONT_INCLUDE_DEBUG) { 
				preg_match_all("/font-family: (.*?);/s", $data, $matches); 
				$rval .= ' <!-- Use: font-family:'.$matches[1][0]. ' -->'.PHP_EOL;
			} else {
				$rval .= PHP_EOL;
			}
			return $rval;
		}
	}

	function read_font_url($url) {
		$ch = curl_init();   
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);   
		curl_setopt($ch, CURLOPT_URL, $url);   
		$res = curl_exec($ch);   
		return $res;
	}
	
	function read_font_file($filename) {
		$rval = file_get_contents($filename);
		return $rval;
	}

	function save_font_file($filename, $content)  {
		file_put_contents($filename, $content);
	}

	function local_name($url) {
		// example: https://fonts.googleapis.com/css?family=Roboto+Condensed:400,300,700'
		if(stripos($url,'family=') !== false ) {
			$myname = substr($url,stripos($url,'family=')+7);
			if(stripos($myname,':') !== false ) { $myname = substr($myname,0,stripos($myname,':'));	}
			if(stripos($myname,'&') !== false ) { $myname = substr($myname,0,stripos($myname,'&'));	}
			$myname = str_replace('+','-',$myname);
			$myname = strtolower ($myname).'.css';
			return $myname;
		}
		return NULL;
	}
	
	function create_folders() {
		if (!file_exists(LOCALFONT_PATH_CSS)) mkdir(LOCALFONT_PATH_CSS, OCTAL_DIR_MODE , false);
		if (!file_exists(LOCALFONT_PATH_FONT)) mkdir(LOCALFONT_PATH_FONT, OCTAL_DIR_MODE , false);
	}
	
	function _localfont_log($message) {
		if(!LOCALFONT_INCLUDE_DEBUG) return;
		$sLogfile = ini_get ('error_log');
		if (!is_writeable($sLogfile)){return false;}
		$sMsg = date('c').' '.'[LOCALFONT] "'.$message.'"'.PHP_EOL;
		file_put_contents($sLogfile, $sMsg, FILE_APPEND);
	}

}
